/********************************************************************
Program:        displaySchedule.sas
Author:         Bill McNeill - SAS Institute
Date:           February 28, 2013
Requirement(s): Base SAS
Purpose:        Create an HTML file that contains a formatted display 
                of the itinerary information collected in all the 
                previous steps. 
Called From:    webDataDemo.sas
Results:        full2013Schedule.html
********************************************************************/
%put ">>>>>>>>>> Entering displaySchedule.sas";
/* Minor name change to the sorted schedule data set. This is needed
   for the merging of the data sets in the next step.  */
data work.finalSchedule(drop=eventDate);
  set work.fullsortedschedule;
  date=eventDate;
run;

/* Add the PERSONAL.WEATHER data set to the WORK.FINALSCHEDULE data 
   set. Remember that the PERSONAL.WEATHER data set is the current
   weather information with the dates modified to the conference 
   dates.*/
data work.dailyReport;
  merge work.finalschedule personal.weather;
  by date;
run;
/*
proc print data=work.dailyReport; run;
*/

/* Assign the output file for the generated ODS HTML output. */ 
ods html(2) path="output/" body="full2013Schedule.html";

/* Display the title of the report. Note that C0362C is the color code 
   for International Orange (Golden Gate Bridge). */
title height=28pt color='#C0362C' "SAS Global Forum 2013";

/* Define the ODS escape character. This allows for inline formatting. */
ods escapechar="^";

/* Display the summary walking directions and map image of the route. */ 
proc report data=work.dirSum nowd split='\' 
            style=[postimage="map.png"];
column totalTime totalDistance hotelAddress sgfAddress walkWarning copyrights;
define totalTime      / noprint;
define totalDistance  / noprint;
define hotelAddress   / noprint;
define sgfAddress     / noprint;
define walkWarning    / noprint;
define copyrights     / noprint;
compute before _page_;
   line "^S={font_weight=bold font_size=12pt}Walking Directions:";
   line "^S={font_weight=bold}From hotel: " hotelAddress $50.; 
   line "^S={font_weight=bold}To conference: " sgfAddress $50.;
   line "^S={font_weight=bold}Distance: " totalDistance $7.;
   line "^S={font_weight=bold}Estimated Time: " totalTime $6.;
   line "";
   line "Warning: " walkWarning $120.;
   line "";
   line copyrights $80.;
   endcomp;
run;

/* Turn off the title so it does not appear between sections of the
   itinerary report. */
title;

/* Display the specific directions for each leg of the route from the
   hotel to the conference. The protectspecialchars=off enables 
   rendering the HTML elements within the direction descriptions 
   (like <B> for bold type). */
proc report data=work.dirSpec nowd;
column time distance directions;
define time       / display width=7 right 'Estimated Time';
define distance   / display width=9 right 'Distance';
define directions / style(column)={protectspecialchars=off} display width=80 left 'Directions';
run;

/* Turn off automatically displaying the date for each day. This is
   done manually with formatting in the REPORT procedure. */
options nobyline;

/* Display the itinerary for each day I'm at the conference. The compute 
   activity section color codes the activities based on the 
   activityType value. The formatted date is produced in the compute
   before _page_ section. */ 
proc report data=work.dailyReport nowd;
by date;
column 	summary mintemp maxtemp morningRainPct eveningRainPct 
        startTimeValue stopTimeValue ActivityType Activity date;
define morningRainPct   / noprint display;
define eveningRainPct   / noprint display;
define summary          / noprint display;
define minTemp          / noprint display;
define maxTemp          / noprint display;
define startTimeValue   / display width=10 right 'Start Time';
define stopTimeValue    / display width=10 right 'End Time';
define activity         / display;
define activityType     / noprint display;
define date             / noprint display;
compute before _page_;
   line "^S={font_weight=bold font_size=16pt}" Date WEEKDATE29.;
   line "";
   line "";
   line "^S={font_weight=bold font_size=12pt}Weather Forecast";
   line "^S={font_weight=bold}" summary $30.;
   line "^S={font_weight=bold}Temperature range: " minTemp 3. " - " maxTemp 3.;
   line "^S={font_weight=bold} Chance of morning rain: "  morningRainPct percent6.
        "^S={font_weight=bold}  Chance of evening rain: " eveningRainPct percent6.;
   line "";
   endcomp;
compute activity;
   if activityType = "Required" then 
      call define(_col_,'style','style=[background=lightred]');
   if activityType = "Desired" then 
      call define(_col_,'style','style=[background=lightgreen]');
   if activityType = "Recommended" then 
      call define(_col_,'style','style=[background=yellow]');
   endcomp;
run;

/* Display the credits for the weather data. */ 
proc report data=work.weatherOverhead nowd;
column title disclaimer credit;
define title      / noprint;
define disclaimer / noprint;
define credit     / noprint;

compute before _page_;
   line "^S={font_weight=bold}Weather data from: " title $75.; 
   line "^S={font_weight=bold}Supplied by: " credit $50.;
   line "^S={font_weight=bold}Disclaimer: " disclaimer $50.;
   endcomp;
run;

/* Close the HTML file contianing the report. */ 
ods html(2) close;

%put "<<<<<<<<<< Exiting displaySchedule.sas";

