/********************************************************************
Program:        xslHtmlTable.sas
Author:         Bill McNeill
Date:           February 28, 2013
Requirement(s): * Base SAS 9.2 or later (except AUTOMAP= option)
                * Base SAS 9.3M2 or later (for AUTOMAP= option)
                * autoWebDemo.sas
                * Defining the htmlUrl macro variable in getHtml.sas.
Purpose:        Demonstrate reading HTML table data into a SAS data 
                set using an XSL transformation via the XSL procedure 
                and the XML LIBNAME engine. 
Results:        SAS data set WORK.XSLSCHEDULE
********************************************************************/
%put ">>>>>>>>>> Entering xslHtmlTable.sas";

/* Get access to the web page. */
filename webIn url "&htmlUrl";

/* Read the source of the HTML page. When the opening <TABLE> element
   is found in the HTML source, begin sending the source lines from 
   the input, webIn, to the output file, justTable.html. When the 
   closing </TABLE> element is found, stop sending the HTML source as
   this signifies the end of the HTML table. */
data _null_;
  /* Boolean noting if in HTML table */
  retain isInTable; 
  /* Assign the output file. */
  file "data\justTable.html";
  /* Assign the input source. */
  infile webIn length=L;
  /* Read a line from the input source. */
  input @1 line $varying32767. L;
  /* Check for the <TABLE> element. */
  if index(line, "<table")  NE 0 then isInTable="YES";
  if isInTable EQ "YES" then do;
    /* The current line of data is from within the HTML table. Copy 
       each line of input to the output file. If found in the input 
       line, convert the following to the desired character. */
	line=tranwrd(line,'&ndash;','-');
    line=tranwrd(line,'&rsquo;',"'");
    line=tranwrd(line,'&nbsp;',' ');
    put @1 line $varying32767. L;
	end;
  /* Check to see if the </TABLE> element is in the current line of 
	 input data. */
  if index(line,"</table") NE 0 then isInTable="NO";
  run;

/* Display the output data file in the SAS log. */
%fileecho("data\justTable.html");

/* Assign a fileref to the input file. */
filename in  'data\justTable.html';
/* Assign a fileref to the XSL stylesheet file. */
filename xsl 'sasuser\htmlTable.xsl';
/* Assign a fileref to the output file. */
filename out 'data\schedule.xml';

/* Transform the HTML table data to XML. */
proc xsl in=in xsl=xsl out=out; run;

/* Display the created XML in the SAS log. */
%fileecho(out);

option source2;
/* Assign a libref for accessing the data in the generated XML file. */
libname sched xmlv2 xmlfileref=out;
/* Failure! Format of data is not supported.
   ERROR: XML data is not in a format supported natively by the XML 
          libname engine. Files of this type may require an XMLMap to 
          be input properly.
   ERROR: Error in the LIBNAME statement.                             */

/* Manually create the file schedule.map using the automapping feature 
   of the XML Mapper application. */

/* Assign a libref for accessing the data in the generated XML file 
   using a XMLMap file, schedule.map. */
libname mapVmapr xmlv2 xmlfileref=out xmlmap="data\schedule.map";
option nosource2;

/* Display the data sets created from the XML data. */
proc datasets lib=mapVmapr; run;
/*
Display the data in the data sets created from the XML data. 
title "activities"; proc print data=mapVmapr.activities; run;
title "event";      proc print data=mapVmapr.event; run;
title "schedule";   proc print data=mapVmapr.schedule; run;
title "table";      proc print data=mapVmapr.table; run;
title "time";       proc print data=mapVmapr.time; run;
*/

option source2;
/* Now use AUTOMAP= of the XMLV2 LIBNAME engine. 
    *** This requires SAS 9.3M2 or later. *** 
   The generated XMLMap file is sent to the temporary device. */
filename tempMap temp;
libname sched xmlv2 xmlfileref=out xmlmap=tempMap automap=replace;
option nosource2;

/* Combine into a single data set the data sets created from the XML 
   data using the XMLMap file. The restriction on the 
   dates.schedule_oridinal is to only include data from conference 
   days Sunday through Wednesday. */
proc sql;
create table work.xslSchedule as (
    select dates.date,
           events.event,
           times.time
    from mapVmapr.schedule   as dates,
         mapVmapr.activities as acts,
         mapVmapr.event      as events,
         mapVmapr.time       as times
    where ( ( (dates.schedule_ordinal GE 3) and
              (dates.schedule_ordinal LE 6)     )               and
            dates.schedule_ordinal  = acts.activities_ordinal   and
            acts.activities_ordinal = events.activities_ordinal and
            acts.activities_ordinal = times.activities_ordinal  and
            times.time_ordinal      = events.event_ordinal)
    );
quit;

/* Display the HTML table data stored in the data set. Note that this
   data still requires additional processing to convert the date and 
   time strings to SAS date and time values. This task is left as an 
   exercise for the reader. */
title justify=center "Raw HTML Table Data via XSL and XML";
proc print data=work.xslSchedule; run;

%put "<<<<<<<<<< Exiting xslHtmlTable.sas";
