/********************************************************************
Program:        getHttpXml.sas
Author:         Bill McNeill - SAS Institute
Date:           February 28, 2013
Requirement(s): * Base SAS 9.3 or later (except AUTOMAP= option)
                * Base SAS 9.3M2 or later (for AUTOMAP= option)
                * autoWebDemo.sas (fileecho macro)
Purpose:        Demonstrate accessing data from a web service using 
                HTTP. The data returned will be in XML. 
Called From:    webDataDemo.sas
Results:        * WORK.DIRSUM
                * WORK.DIRSPEC
********************************************************************/
%put ">>>>>>>>>> Entering getHttpXml.sas";

/* Assign the fileref for the output file. */
filename directs 'data\directions.xml';

%fileecho(directs);
filename tempMap Temp;

/* Assign the libref to use the XMLV2 LIBNAME engine to place the XML
   data into SAS data sets. 
   *** This requires SAS 9.3M2 or better. *** 
   If not available, the XMLMap file needs to be create with the XML 
   Mapper application. The XMLMap file only needs to be autogenerated. */
libname walkDirs xmlv2 xmlfileref=directs xmlmap=tempMap automap=replace;
/* Display the data sets created from the XML data using the XMLMap file. */
proc datasets lib=walkDirs; run;

/*
Display the data in the data sets. 

title "duration (legs)";                       proc print data=directs.duration; run;
title "duration1 (total time)";                proc print data=directs.duration1; run;
title "distance (legs)";                       proc print data=directs.distance; run;
title "start_location (legs)";                 proc print data=directs.start_location; run;
title "end_location (legs)";                   proc print data=directs.end_location; run;
title "start_location1 (hotel - lat & long)";  proc print data=directs.start_location1; run;
title "end_location1 (conf - lat & long)";     proc print data=directs.end_location1; run;
title "route (summary, copyright, & warning)"; proc print data=directs.route; run;
title "directions response (GET status)";      proc print data=directs.directionsresponse; run;
title "leg (start and end addresses)";         proc print data=directs.leg; run;
title "northeast (lat & long??)";              proc print data=directs.northeast; run;
title "step (directions in html)";             proc print data=directs.step; run;
*/

/* Using the ordinal values in the data sets, combine all the desired 
   data for the summary of the directions into a single data set,
   WORK.DIRSUM. */
proc sql;
create table work.dirSum as (
   select dur.text as totalTime,
          dis.text as totalDistance,
          rou.copyrights,
		  rou.warning as walkWarning,
		  leg.start_address as hotelAddress,
		  leg.end_address   as sgfAddress
   from walkDirs.duration1 as dur,
        walkDirs.distance1 as dis,
		walkDirs.route     as rou,
		walkDirs.leg       as leg);
   quit;
title "directions summary";
proc print data=dirSum; run;

/* Use the same technique as in the last step to create a data set of
   the specific directions. */
proc sql;
create table work.dirSpec as (
   select step.step_ordinal      as leg,
          dur.text               as time,
          dis.text               as distance,
          start.lat              as startLatitude,
          start.lng              as startLongitude,
          end.lat                as endLatitude,
          end.lng                as endLongitude,
          step.html_instructions as directions length=200
   from (walkDirs.duration       as dur,
         walkDirs.distance       as dis,
         walkDirs.start_location as start,
         walkDirs.end_location   as end,
         walkDirs.step           as step)
   where (dur.step_ordinal = dis.step_ordinal   and 
          dur.step_ordinal = start.step_ordinal and 
          dur.step_ordinal = end.step_ordinal   and
          dur.step_ordinal = step.step_ordinal      )
   );
   quit;
proc contents data=dirSpec; run;
title "Direction specifics";
proc print data=dirSpec; run;

/* For this step, a more complex XMLMap file needs to be created. The
   detailed directions for creating the XMLMap file are in the paper. 
   Once that XMLMap file is created, the libref can be assigned and, 
   using the created XMLMap file, the data from the data sets can be 
   combined in one. The XMLMap file directs the XMLV2 LIBNAME engine
   on how to combine the data sets. Due to using this XMLMap file, no
   SQL procedure is needed (in contrast to the previous step where it
   was needed). */
libname walkDir2 xmlv2 xmlfileref=directs xmlmap="data\directions.map";

/* Display the specific directions data set as created from the XMLMap
   file. It should be the same as the data set created in the previous 
   step. */
title "Direction specifics (from XMLMap file)";
proc print data=walkDir2.dirSpec; run;


/* Route Map Image */
/* With the directions for each leg of the journey obtained, the 
   information for creating the map image is available. As above,
   a URL needs to be created containing the needed data to create
   the map image. This means inserting into the URL the starting 
   and ending latitude and longitude for each leg. This part of the 
   URL will be created in a DATA step. 
   The form of this part of the URL is:
   &path=start_latitude,start_longitude|end_latitude,end_longitude<|
   start_latitude_2,start_longitude_2|end_latitude_2,end_longitude_2...> */
data _null_;
set walkDir2.dirSpec(keep=leg startLatitude startLongitude
                              endLatitude   endLongitude)
            end=done; /* Where to go when teh data is exhausted. */
   retain fullPath;  /* Keep the full path value throughout iterations. */

   /* Concatenate the start and end coordinates for the current leg. */
   startLocation=cats(put(startLatitude,12.5), ",",
                      put(startLongitude,12.5));
   endLocation=cats(put(endLatitude,12.5), ",",
                    put(endLongitude,12.5));
   legPath=cats(startLocation, "|", endLocation);

   /* On the first iteration, start the string with "&path=". On all
      other iterations, append "|" and then the coordinates string for
      the current leg.  */ 
   if _n_ eq 1 
      then fullPath=cats('&path=',legPath);
      else fullPath=cats(%nrstr(fullPath),"|",legPath);
   /* When the data is done, put the generated path value into the 
	  macro variable "mapPath". */
   if done then call symput("mapPath",%nrstr(fullPath));
run;

/* Display in the SAS log the assembled mapPath string. 
%put %superq(mapPath);
*/

%put "<<<<<<<<<< Exiting getHttpXml.sas";
