/********************************************************************
Program:        getSoapXml.sas
Author:         Bill McNeill - SAS Institute
Date:           February 28, 2013
Requirement(s): * Base SAS 9.3 or later
                * WORK.DIRSPEC created in getHttpXml.sas
                * autoWebDataDemo.sas (fileecho macro)
Purpose:        Demonstrate accessing data from a web service using 
                SOAP. The data returned will be in XML. 
Called From:    webDataDemo.sas
Results:        * WORK.WEATHER
                * WORK.WEATHEROVERHEAD
Note:           It is not possible in March to get weather forecasts 
                for the end of April. Therefore, the forecast data
                is for the current date and the next 3 days. The dates
                are later modified in personalSchedule.sas to the 
                conference dates.
********************************************************************/
%put ">>>>>>>>>> Entering getSoapXml.sas";
/* Using the WORK.DIRSPEC data set created in getHttpXml.sas, assign
   the hotel's latitude and longitude values in macro variables. Also
   get the current date string. These macro variables are used in the
   requesting SOAP message.                                           */
data _null_;
  set work.dirSpec;
  if _n_=1 then do; /* The first coordinate pair is the hotel. */
     call symput("latitude",  compress(put(startLatitude,  12.5)));
     call symput("longitude", compress(put(startLongitude, 12.5)));
     call symput("date", put(today(), mmddyy10.));
	 end;
  run;

/* Get the weather forecasts for the 4 days. */
%let numberOfDays=4;

/* Assign the filerefs for the SOAP request and response messages. */
filename request  "output\soapInput.xml";
filename response "output\soapOutput.xml";

/* Create the SOAP request message. There are 6 parameters needed for 
   this message: 
      1) latitude
      2) longitude
      3) start date of forecasts
      4) number of days of forecasts
      5) units type of the forecasts data
      6) forecast type
   The first 4 values are included in the request message via macro
   variables. The last 2 parameters are hardcoded into the message.
*/
data _null_; 
     input; 
     line=resolve(_infile_); /* Resolves macro values in datalines. */
     resolveLen=length(line); 
     file request; 
     put @1 line $varying200. resolveLen; 
     cards4; 
<soapenv:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
                  xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
                  xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" 
                  xmlns:ndf="http://graphical.weather.gov/xml/DWMLgen/wsdl/ndfdXML.wsdl">
   <soapenv:Header/>
   <soapenv:Body>
      <ndf:NDFDgenByDay soapenv:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
         <latitude xsi:type="xsd:decimal">&latitude</latitude>
         <longitude xsi:type="xsd:decimal">&longitude</longitude>
         <startDate xsi:type="xsd:date">&date</startDate>
         <numDays xsi:type="xsd:integer">&numberOfDays</numDays>
         <Unit xsi:type="dwml:unitType" xmlns:dwml="http://graphical.weather.gov/xml/DWMLgen/schema/DWML.xsd">e</Unit>
         <format xsi:type="dwml:formatType" xmlns:dwml="http://graphical.weather.gov/xml/DWMLgen/schema/DWML.xsd">24 hourly</format>
      </ndf:NDFDgenByDay>
   </soapenv:Body>
</soapenv:Envelope>
;;;;
run; 

/* Display the SOAP request message to the SAS log. 
%fileecho(request);
*/

/* Send the SOAP request message to the proper URL with the proper 
   SOAP action specified. The desired SOAP action was determined by 
   reviewing the choices at http://graphical.weather.gov/xml. The 
   specific details of the SOAP action were obtained from 
   http://graphical.weather.gov/xml/SOAP_server/ndfdXMLserver.php and
   selecting the NDFDgenByDay link.

   Note that in my case I needed to use a proxy host and port. If your 
   site demands this, you will need to supply your site specific values.
   For additional details see the Base SAS Procedures Guide.            */
/*
proc soap in=request
          out=response
          url="http://graphical.weather.gov/xml/SOAP_server/ndfdXMLserver.php"
          soapaction="http://graphical.weather.gov/xml/DWMLgen/wsdl/ndfdXML.wsdl#NDFDgenByDay"
          ;
run;  
*/
/* Display the SOAP response message to the SAS log. If there are errors
   in the request of the data, the error message(s) will be contained in
   this file. 
%fileecho(response);
*/

/* In some cases, the XML needs to be cleaned up a bit. Here, there
   were lines at the top and bottom of the file that the XMLV2 LIBNAME
   engine did not like, so they were stripped here. Also, replace the 
   character entity references (like &lt;) with the corresponding 
   character. */
%let xmlFile=output\sfWeather.xml;
data _null_;
  retain isInXML;
  file "&xmlFile";
  infile response length=L;
  input @1 line $varying32767. L;
  /* Anything before the <?xml element is ignored. */
  xmlIndex = index(line,'&lt;?xml');
  if xmlIndex NE 0 
     then do;
       isInXML="YES";
	   line = substr(line, xmlIndex);
	   end;
  if isInXML EQ "YES" then do;
	line=tranwrd(line,'&ndash;', '-');
    line=tranwrd(line,'&rsquo;', "'");
    line=tranwrd(line,'&nbsp;',  ' ');
	line=tranwrd(line,'&lt;',    '<');
	put   @1 line $varying32767. L;
	end;
  if index(line,"</dwml") NE 0 then isInXML="NO";
  run;

/* Display the final form of the XML file in the SAS log. */
%fileecho("&xmlFile");
  
/* Create an XMLMap file via the automapping feature of the XML Mapper
   application. Save the XMLMap file to data\sanFranWeather.map */
   
filename wthrMap "data\sanFranWeather.map";
libname sfWeathr xmlv2 "&xmlFile" xmlmap=wthrMap;

/* Show the VALUE data set to identify the additional work needed to
   get the data into the proper format.                              */
proc print data=sfWeathr.value; run;

/* Due to the way the data is presented in the XML file the 
   value_ordinal is not in the form needed. The 
   temperature_ordinal value identifies the low and high 
   temperature values. The low temperature values need to have the
   value_ordinal reset to 1-4. This DATA step resets the low 
   temperature value_ordinal vales.                                */
data work.resetOrdinal;
  set sfWeathr.value;
  if value_ordinal > &numberOfDays
     then value_ordinal = value_ordinal - &numberOfDays;
run;

/*
proc print data=work.resetOrdinal; run;
*/


/* Sort the high/low temperature data set by day (as indicated by the
   value_ordinal value). */ 
proc sort data=work.resetOrdinal out=work.sortedOrdinal;
  by value_ordinal temperature_ordinal;
run;

/*
proc print data=work.sortedOrdinal; run;
*/

/* Make a data set with the high and low temperature values in 
   descriptive variables. */
data work.minMaxTemps(drop=value temperature_ordinal);
  set work.sortedOrdinal;
  retain maxTemp;
  if temperature_ordinal = 1 
     then maxTemp = value;
     else do; 
       minTemp = value;
	   output; /* Observation of daily high and low temperatures. */
	   end;
run;

/*
proc print data=work.minMaxTemps; run;
*/

/* A similiar problem exists with the probability of rain data. In 
   this case, the odd value1_ordinal values are for morning percentage and 
   the even value1_ordinal values are for evening percentage. Also, convert
   the value to a percentage by dividing by 100. */
proc print data=sfWeathr.value1; run;
data work.mornEvenRain(drop= value1 probability_of_precipitation_ord 
                             value1_ordinal);
  set sfWeathr.value1;
  retain morningRainPct ordinal;
  if mod(value1_ordinal,2) = 0  /* Even value for value1_ordinal */
     then do; 
       eveningRainPct = (value1 / 100);
	   ordinal+1; 
	   output; /* Output observation of morning and evening percentage. */
	   end;
     else morningRainPct = (value1 /100);
run;

/*
proc print data=work.mornEvenRain; run;
*/

title "temperature";        proc print data=sfWeathr.temperature;        run;
title "time_layout";        proc print data=sfWeathr.time_layout;        run;
title "start_valid_time";   proc print data=sfWeathr.start_valid_time;   run;
title "weather_conditions"; proc print data=sfWeathr.weather_conditions; run;

/* Combine all the desired daily weather data into a single data set
   with an observation for each day. */
proc sql;
create table work.weather as (
	select startTime.start_valid_time as date, 
		   tempValues.maxTemp,
		   tempValues.minTemp,
		   description.weather_conditions_weather_summa as summary,
		   rain.morningRainPct,
           rain.eveningRainPct
	from sfWeathr.temperature as temp,
		 sfWeathr.time_layout as timeType,
         sfWeathr.start_valid_time as StartTime,
		 work.minMaxTemps as tempValues,
		 sfWeathr.weather_conditions as description,
		 work.mornEvenRain as rain
	where temp.temperature_time_layout1      = timeType.layout_key and
          startTime.start_valid_time_ordinal = tempValues.value_ordinal and 
          startTime.start_valid_time_ordinal = description.weather_conditions_ordinal and
          startTime.start_valid_time_ordinal = rain.ordinal and
          startTime.time_layout_ordinal      = time_layout.time_layout_ordinal and
		  /* Because the high/low temperatures were reformatted into 
		     the same observation, unless the temp.temperature_ordinal 
		     is restricted to one record, data will be duplicated.     */
		  temp.temperature_ordinal = 1
    )
	order by date 
;
title "Forecasted Weather";
proc print data=work.weather; run;

/* Combine all the desired summary weather data into a single data set. */
data work.weatherOverhead(keep=title disclaimer credit credit_logo); 
merge sfWeathr.product 
      sfWeathr.source; 
	  by head_ordinal;
run;
title "Forecasted Weather Overhead";
proc print data=work.weatherOverhead; run;

%put "<<<<<<<<<< Exiting getSoapXml.sas";
