/********************************************************************
Program:        processHtmlTable.sas
Author:         Bill McNeill - SAS Institute
Date:           February 28, 2013
Requirement(s): Base SAS
                WORK.TABLE1 (from readHtmlFile.sas)
Purpose:        Process a data set in a known format into a different 
                format more conducive to the desired output for display.
Called from:    getHtml.sas
Produces:       SAS data set WORK.SCHEDULE
********************************************************************/
%put ">>>>>>>>>> Entering processHtmlTable.sas";
/*------------------------------------------------------------------
Name:       getTimeValue
Purpose:    Convert a time string to a SAS time. The time string is
            in the form:
                #<#><:##><a.m.|p.m.>
                where:
                  #  is a single digit number
                  <> indicates the enclosed entity is optional.
                  |  indicates a logical OR condition
Parameters: timeString - The name of the variable that contains the
                         string of the time to be converted to a 
                         SAS time value.
            timeValue - The name of the variable that will contain 
                        the time as a SAS time value. 
-------------------------------------------------------------------*/
%macro getTimeValue(timeString, timeValue);
   /* Extract the single or double digit numeric time value. */
   blankIndex = index(&timeString, ' ');
   timeNumber = left(substr(&timeString, 1, blankIndex));

   /* Attempt to extract the meridian indicator. It may not be present. */
   meridianString = left(compress(substr(&timeString, blankIndex)));
   /* The TIME informat wants either AM or PM. Convert "a.m." or "p.m." 
      to the proper value in the meridian string. */
   meridianString = transtrn(meridianString, "a.m.", "AM");
   meridianString = transtrn(meridianString, "p.m.", "PM");

   /* The TIME informat would like a colon seperating the hour value
      from the minute value (seconds are optional). If a colon is not 
      present in the time value string, add it with "00" minutes. */
   colonIndex = index(timeNumber, ':');
   if colonIndex EQ 0 
      then timeNumber = catx(':', timeNumber, "00");

   /* See if the meridian indicator is present in the meridian string.
	  If not, use the passed in isAfterMidday value to set the 
	  proper meridian value. */
   meridianIndex = index(meridianString, 'M');
   if meridianIndex EQ 0 then do;
      if isAfterMidday EQ 0 
         then timeNumber = catx(' ', timeNumber, "AM");
         else timeNumber = catx(' ', timeNumber, "PM");
      end;
   else
     timeNumber = catx(' ', timeNumber, meridianString);
   /* Convert the time number string to a SAS time value in the TIME8. 
	  format. This value is returned to the calling macro. */ 
   &timeValue = input(timeNumber, TIME8.);
%mend  getTimeValue;


/*------------------------------------------------------------------
Name:       getTimes
Purpose:    Convert a time span string to start and stop times. The 
            start and stop times will be SAS time values.
            The time string is in the form:
                #<#><:##><a.m.|p.m.> - #<#><:##>a.m.|p.m.
                where:
                  #  is a single digit number
                  <> indicates the enclosed entity is optional.
                  |  indicates a logical OR condition
Parameters: columnValue - The name of the variable that contains the 
                          string of the time to be converted to SAS
                          time values.
-------------------------------------------------------------------*/
%macro getTimes(columnValue);
     /* Get the string after the hyphen, '-', the stop time string. */
     stopTimeIndex      = index(&columnValue, '-') + 1;
     stopTimeString     = trim(left(substr(&columnValue,(stopTimeIndex + 1))));

	 /* Get the string before the hyphen, the start time string. */
     startTimeString    = left(substr(&columnValue, 1,(stopTimeIndex - 2)));

	 /* The stopTimeString always includes the meridian indicator 
	    (AM or PM) for the time. Based on this indicator, set a 
	    variable noting the indicator found. This is used when the 
	    start time does not have a meridian indicator. This case 
	    implicitly notes that the start time and end time are in the 
	    same meridian.                                               */
	 qualifiedTimeIndex = index(stopTimeString, "p.m.");
	 if qualifiedTimeIndex GT 1 
        then isAfterMidday = 1;
        else isAfterMidday = 0;

	 /* Get the SAS time values based on the start and stop time string. */
	 %getTimeValue(stopTimeString,  stopTimeValue); 
	 %getTimeValue(startTimeString, startTimeValue); 
%mend  getTimes;

/*------------------------------------------------------------------
Name:       getEventDate
Purpose:    Convert a date string to a SAS date value.
            The date string is in the form:
                full_name_day_of_week, full_month_name date_number
                where:
                  full_name_day_of_week is "Monday", "Tuesday", etc.
                  full_month_name is, in this case, "April" or "May".
                  date_number is a single or double digit date number.
-------------------------------------------------------------------*/
%macro getEventDate();
     /* Get the first letter of the month. This should be 'A' or 'M'. */
     monthIndex  = index(col1, ',') + 2;
	 monthStr    = left(substr(col1, monthIndex));
     monthLetter = substr(monthStr, 1, 1);
	 /* Based on the month letter, get the number of the month and 
	    the length of the month string. */
	 if monthLetter = 'A' then do;
	 	month = "04";
		length = length("April");
		end;
     else do;
		month = "05";
		length = length("May");
		end;
	/* Get the day number and make two digits if necessary. */
	dayNumber = trim(substr(monthStr, length + 2));
	if length(dayNumber) = 1 then dayNumber = "0" || dayNumber;

	/* Set the needed constants for the date string to be converted. 
	   The date string is in the form of DD/MM/YYYY. */
	yearString = "2013";
	slash='/';
	eventDateString = catx(slash, month, dayNumber, yearString);
	/* Convert the date string to a SAS date value. */
	eventDate = input(eventDateString, mmddyy10.);
%mend  getEventDate;


/*
option mprint;
*/


/* Read through the observations in the WORK.TABLE1 data set. Save 
   the observations for activities inclusive of Sunday through 
   Wednesday. For the observations matching those days, convert the
   date or time strings in COL1 to SAS date or time values. Add the 
   activityType field to the data set and fill with the value "General". 
   The desired data is save in the WORK.SCHEDULE data set.
*/
data work.schedule(keep=eventDate activity activityType 
                        startTimeValue stopTimeValue);
/* Make activityType big enough to hold all the eventual values. */
length activityType $12;
set work.table1;
format  startTimeValue timeampm8.
		stopTimeValue  timeampm8.
		eventDate      WEEKDATE29.;
retain eventDate;
/* Get the first 3 characters of the value in COL1. See if the column
   data is a date of interest. */
startOfCol=substr(col1,1,3);
if startOfCol = "Sun" or
   startOfCol = "Mon" or
   startOfCol = "Tue" or
   startOfCol = "Wed" 
   then do;
     /* The column value is a date. Convert the date string to a SAS 
        date value. Save the converted value for the next 
        observation(s). */
     %getEventDate();
	 /* With the SAS date value retained for the following observations,
	    do not save this observation -- no more useful information. */
	 delete;
	 end;
   else do;
     /* The column data could contain desired information. Column 1 
        can contain the activity date. Column 3 can contain the 
        activity description. If either of these columns are empty or
        the eventDate is not set, the observation is not of interest. */
	 if length(Col1) eq 1 or length(Col3) eq 1 or eventDate eq .
        then delete; /* nothing interesting - do not save obs. */
	    else do;
		  /* There is data of interest in the observation. Process the
		     data in the observation. */
		  activity     = col3; /* Saving as "activity" in new data set. */
		  activityType = "General"; /* New colunmn */
		  /* Get the time string and convert to SAS time value for the 
		     proper start and end time of the activity. */ 
		  %getTimes(col1);
		  /* Save the desired columns in the SCHEDULE data set. */
          output;
		  end; /* else valid activity data */
	    end; /* else activity data */
run;

/* Display the created WORK.SCHEDULE data set. */ 
title  justify=center "Schedule data";
proc print data=schedule; run;

%put "<<<<<<<<<< Exiting processHtmlTable.sas";
